//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2001, 2002, 2003 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: BoundaryConditionDiscretization.hpp,v 1.4 2005/11/20 21:24:42 delpinux Exp $


#ifndef BOUNDARY_CONDITION_DISCRETIZATION_HPP
#define BOUNDARY_CONDITION_DISCRETIZATION_HPP

#include <ReferenceCounting.hpp>

#include <BaseMatrix.hpp>
#include <Vector.hpp>

#include <Problem.hpp>
#include <DegreeOfFreedomSet.hpp>

/**
 * @file   BoundaryConditionDiscretization.hpp
 * @author Stephane Del Pino
 * @date   Sun Nov 24 17:37:59 2002
 * 
 * @brief  
 * 
 */

class BoundaryConditionDiscretization
{
protected:
  const Problem& __problem;

  const DegreeOfFreedomSet& __degreeOfFreedomSet;

  //! true for vertices supporting Dirichlet.
  mutable Vector<bool> __dirichletList;

  mutable Vector<real_t> __dirichletValues;

public:
  const bool& dirichlet(const size_t i) const
  {
    return __dirichletList[i];
  }

  const real_t& dirichletValue(const size_t i) const
  {
    return __dirichletValues[i];
  }

  const Vector<bool>& getDirichletList() const
  {
    return __dirichletList;
  }

  virtual void getDiagonal (BaseVector& X) const = 0;

  virtual void setMatrix (ReferenceCounting<BaseMatrix> A,
			  ReferenceCounting<BaseVector> b) const = 0;

  virtual void setSecondMember (ReferenceCounting<BaseMatrix> A,
				ReferenceCounting<BaseVector> b) const = 0;

  virtual void timesX(const BaseVector& X, BaseVector& Z) const = 0;

  virtual void transposedTimesX(const BaseVector& X, BaseVector& Z) const = 0;

  const Problem& problem() const
  {
    return __problem;
  }

  BoundaryConditionDiscretization(const Problem& problem,
				  const DegreeOfFreedomSet& dof)
    : __problem(problem),
      __degreeOfFreedomSet(dof),
      __dirichletList(dof.size())
  {
    __dirichletList = false;
  }

  virtual ~BoundaryConditionDiscretization()
  {
    ;
  }
};

#endif // BOUNDARY_CONDITION_DISCRETIZATION_HPP
