//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2005 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: DiscretizationType.hpp,v 1.4 2006/11/24 00:02:23 delpinux Exp $

#ifndef DISCRETIZATION_TYPE_HPP
#define DISCRETIZATION_TYPE_HPP

#include <ErrorHandler.hpp>
#include <string>

/**
 * @file   DiscretizationType.hpp
 * @author Stephane Del Pino
 * @date   Mon May 30 23:28:37 2005
 * 
 * @brief  This class describes type of Discretization
 * 
 */
class DiscretizationType
{
public:
  enum Type {
    LagrangianFEM0 = 0,		/**< Lagrangian finite element of degree 0 */
    LagrangianFEM1 = 1,		/**< Lagrangian finite element of degree 1 */
    LagrangianFEM2 = 2		/**< Lagrangian finite element of degree 2 */
  };

private:
  const Type __type;		/**< Type of discretization */

public:

  /** 
   * convertes type in its name
   * 
   * @param type type of discretization
   * 
   * @return name of the type 
   */
  static std::string name(const DiscretizationType::Type& type)
  {
    switch (type) {
    case LagrangianFEM0: return "P0";
    case LagrangianFEM1: return "P1";
    case LagrangianFEM2: return "P2";
    }
    throw ErrorHandler(__FILE__,__LINE__,
		       "unknown FEM type",
		       ErrorHandler::unexpected);
  }

  /** 
   * Read-only access to the type
   * 
   * @return __type
   */
  const Type& type() const
  {
    return __type;
  }

  /** 
   * Constructor
   * 
   * @param type type of discretization
   */
  DiscretizationType(const DiscretizationType::Type& type = LagrangianFEM1)
    : __type(type)
  {
    ;
  }

  /** 
   * Copy constructor
   * 
   * @param d originale discretization type
   */
  DiscretizationType(const DiscretizationType& d)
    : __type(d.__type)
  {
    ;
  }

  /** 
   * Destructor
   * 
   */
  ~DiscretizationType()
  {
    ;
  }
};

#endif // DISCRETIZATION_TYPE_HPP
