//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2001, 2002, 2003 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: FieldFunctionWriterBase.cpp,v 1.2 2006/11/22 23:47:55 delpinux Exp $

#include <FieldFunctionWriterBase.hpp>

#include <ScalarFunctionBase.hpp>
#include <FEMFunctionBase.hpp>

#include <Mesh.hpp>


void FieldFunctionWriterBase::
__fillCrossedComponent(Vector<real_t>& values) const
{
  assert(values.size() == __mesh->numberOfVertices());

  // First component
  switch (__f1->type()) {
  case ScalarFunctionBase::femfunction: {
    const FEMFunctionBase& fem
      = static_cast<const FEMFunctionBase&>(*__f1);
    if ((fem.baseMesh() == __mesh) and
	((fem.discretizationType() == DiscretizationType::LagrangianFEM1) or
	 (fem.discretizationType() == DiscretizationType::LagrangianFEM1))) {
      for (size_t i=0; i<fem.values().size(); ++i) {
	values[3*i] = fem[i];
      }
      break;
    } // if not continues the standard method
  }
  default: {
    const ScalarFunctionBase& f = *__f1;
    for (size_t i=0; i<__mesh->numberOfVertices(); ++i) {
      const TinyVector<3,real_t>& X = __mesh->vertex(i);
      values[3*i] = f(X);
    }
  }
  }

  // Second component
  switch (__f2->type()) {
  case ScalarFunctionBase::femfunction: {
    const FEMFunctionBase& fem
      = static_cast<const FEMFunctionBase&>(*__f2);
    if ((fem.baseMesh() == __mesh) and
	((fem.discretizationType() == DiscretizationType::LagrangianFEM1) or
	 (fem.discretizationType() == DiscretizationType::LagrangianFEM1))) {
      for (size_t i=0; i<fem.values().size(); ++i) {
	values[3*i+1] = fem[i];
      }
      break;
    } // if not continues the standard method
  }
  default: {
    const ScalarFunctionBase& f = *__f2;
    for (size_t i=0; i<__mesh->numberOfVertices(); ++i) {
      const TinyVector<3,real_t>& X = __mesh->vertex(i);
      values[3*i+1] = f(X);
    }
  }
  }

  // Third component
  switch (__f3->type()) {
  case ScalarFunctionBase::femfunction: {
    const FEMFunctionBase& fem
      = static_cast<const FEMFunctionBase&>(*__f3);
    if ((fem.baseMesh() == __mesh) and
	((fem.discretizationType() == DiscretizationType::LagrangianFEM1) or
	 (fem.discretizationType() == DiscretizationType::LagrangianFEM1))) {
      for (size_t i=0; i<fem.values().size(); ++i) {
	values[3*i+2] = fem[i];
      }
      break;
    } // if not continues the standard method
  }
  default: {
    const ScalarFunctionBase& f = *__f3;
    for (size_t i=0; i<__mesh->numberOfVertices(); ++i) {
      const TinyVector<3,real_t>& X = __mesh->vertex(i);
      values[3*i+2] = f(X);
    }
  }
  }
}

FieldFunctionWriterBase::
FieldFunctionWriterBase(ConstReferenceCounting<ScalarFunctionBase> f1,
			ConstReferenceCounting<ScalarFunctionBase> f2,
			ConstReferenceCounting<ScalarFunctionBase> f3,
			ConstReferenceCounting<Mesh> mesh,
			const std::string& fileName,
			const std::string& CR)
  : __f1(f1),
    __f2(f2),
    __f3(f3),
    __mesh(mesh),
    __fileName(fileName),
    __CR(CR)
{
  ;
}

FieldFunctionWriterBase::
~FieldFunctionWriterBase()
{
  ;
}
