//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2001, 2002, 2003 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: FieldFunctionWriterBase.hpp,v 1.2 2006/11/22 23:47:55 delpinux Exp $

#ifndef FIELD_FUNCTION_WRITER_BASE_HPP
#define FIELD_FUNCTION_WRITER_BASE_HPP

#include <ReferenceCounting.hpp>
#include <Vector.hpp>

#include <string>

class ScalarFunctionBase;
class Mesh;

/**
 * @file   FieldFunctionWriterBase.hpp
 * @author Stephane Del Pino
 * @date   Mon Aug 21 19:24:40 2006
 * 
 * @brief  base class for field writer objects
 */
class FieldFunctionWriterBase
{
protected:
  ConstReferenceCounting<ScalarFunctionBase>
  __f1;				/**< the first component */
  ConstReferenceCounting<ScalarFunctionBase>
  __f2;				/**< the second component */
  ConstReferenceCounting<ScalarFunctionBase>
  __f3;				/**< the third component */
  ConstReferenceCounting<Mesh>
  __mesh;			/**< the saving mesh */
  const std::string __fileName;	/**< name of the saving mesh */
  const std::string __CR;	/**< type of cardriage return */

  /** 
   * Fills crossed component
   * 
   * @param values values vector
   */
  void __fillCrossedComponent(Vector<real_t>& values) const;

private:
  /** 
   * Forbidden copy constructor
   * 
   * @param writer given writer
   */
  FieldFunctionWriterBase(const FieldFunctionWriterBase& writer);

public:
  /** 
   * writes the function
   * 
   */
  virtual void proceed() const=0;

  /** 
   * Constructor
   * 
   * @param f1 the first component of the field
   * @param f2 the second component of the field
   * @param f3 the third component of the field
   * @param mesh the mesh used for saving
   * @param fileName the name of the file
   * @param CR the type of carriage-return sequence
   */
  FieldFunctionWriterBase(ConstReferenceCounting<ScalarFunctionBase> f1,
			  ConstReferenceCounting<ScalarFunctionBase> f2,
			  ConstReferenceCounting<ScalarFunctionBase> f3,
			  ConstReferenceCounting<Mesh> mesh,
			  const std::string& fileName,
			  const std::string& CR);

  /** 
   * Destructor
   * 
   */
  virtual ~FieldFunctionWriterBase();
};

#endif // FIELD_FUNCTION_WRITER_BASE_HPP
