//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2001, 2002, 2003 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: FieldFunctionWriterMedit.cpp,v 1.2 2006/11/22 23:47:55 delpinux Exp $

#include <FieldFunctionWriterMedit.hpp>

#include <ScalarFunctionBase.hpp>
#include <FEMFunctionBase.hpp>

#include <Mesh.hpp>

#include <fstream>

void FieldFunctionWriterMedit::
proceed() const
{
  std::string fileName = __fileName;
  fileName += ".bb";
  std::ofstream file(fileName.c_str());
  if (not(file)) {
    throw ErrorHandler(__FILE__,__LINE__,
		       "cannot open file '"+stringify(fileName)+"'",
		       ErrorHandler::normal);
  }

  file << "3 3 " << __mesh->numberOfVertices() << " 2" << __CR;

  Vector<real_t> values(3*__mesh->numberOfVertices());

  this->__fillCrossedComponent(values);

  for (size_t i=0; i<values.size(); ++i) {
    file << values[i] << __CR;
  }
}

FieldFunctionWriterMedit::
FieldFunctionWriterMedit(ConstReferenceCounting<ScalarFunctionBase> f1,
			 ConstReferenceCounting<ScalarFunctionBase> f2,
			 ConstReferenceCounting<ScalarFunctionBase> f3,
			 ConstReferenceCounting<Mesh> mesh,
			 const std::string& fileName,
			 const std::string& CR)
  : FieldFunctionWriterBase(f1, f2, f3, mesh, fileName, CR)
{
  ;
}

FieldFunctionWriterMedit::
~FieldFunctionWriterMedit()
{
  ;
}
