//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2006 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: ScalarFunctionReaderBuilder.cpp,v 1.2 2006/09/20 16:46:33 delpinux Exp $

#include <ScalarFunctionReaderBuilder.hpp>

#include <ScalarFunctionReaderBase.hpp>

#include <ScalarFunctionReaderMedit.hpp>
#include <ScalarFunctionReaderRaw.hpp>

#include <Mesh.hpp>

ReferenceCounting<ScalarFunctionReaderBase>
ScalarFunctionReaderBuilder::
getReader() const
{
  switch (__fileDescriptor.format()) {
  case FileDescriptor::medit: {
    return new ScalarFunctionReaderMedit(__filename, __mesh);
  }
  case FileDescriptor::raw: {
    return new ScalarFunctionReaderRaw(__filename, __mesh);
  }
  default: {
    fferr(0) << __FILE__ << ':' << __LINE__ << ": Not implemented\n";
    std::exit(1);
    return 0;
  }
  }
}

ScalarFunctionReaderBuilder::
ScalarFunctionReaderBuilder(const std::string& filename,
			    ConstReferenceCounting<Mesh> mesh,
			    const FileDescriptor& fileDescriptor)
  : __filename(filename),
    __mesh(mesh),
    __fileDescriptor(fileDescriptor)
{
  ;
}

ScalarFunctionReaderBuilder::
ScalarFunctionReaderBuilder(const ScalarFunctionReaderBuilder& builder)
  : __filename(builder.__filename),
    __mesh(builder.__mesh),
    __fileDescriptor(builder.__fileDescriptor)
{
  ;
}

ScalarFunctionReaderBuilder::
~ScalarFunctionReaderBuilder()
{
  ;
}
