//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2001, 2002, 2003 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: ScalarFunctionWriterBase.hpp,v 1.1 2006/08/27 21:40:33 delpinux Exp $

#ifndef SCALAR_FUNCTION_WRITER_BASE_HPP
#define SCALAR_FUNCTION_WRITER_BASE_HPP

#include <ReferenceCounting.hpp>
#include <string>

class ScalarFunctionBase;
class Mesh;

/**
 * @file   ScalarFunctionWriterBase.hpp
 * @author Stephane Del Pino
 * @date   Mon Aug 21 19:24:40 2006
 * 
 * @brief  base class for function writer objects
 */
class ScalarFunctionWriterBase
{
protected:
  ConstReferenceCounting<ScalarFunctionBase>
  __function;			/**< the function to write */
  ConstReferenceCounting<Mesh>
  __mesh;			/**< the saving mesh */
  const std::string __fileName;	/**< name of the saving mesh */
  const std::string __CR;	/**< type of cardriage return */

private:
  /** 
   * Forbidden copy constructor
   * 
   * @param writer given writer
   */
  ScalarFunctionWriterBase(const ScalarFunctionWriterBase& writer);

public:
  /** 
   * writes the function
   * 
   */
  virtual void proceed() const=0;

  /** 
   * Constructor
   * 
   * @param function the function to save
   * @param mesh the mesh used for saving
   * @param fileName the name of the file
   * @param CR the type of carriage-return sequence
   */
  ScalarFunctionWriterBase(ConstReferenceCounting<ScalarFunctionBase> function,
			   ConstReferenceCounting<Mesh> mesh,
			   const std::string& fileName,
			   const std::string& CR);

  /** 
   * Destructor
   * 
   */
  virtual ~ScalarFunctionWriterBase();
};

#endif // SCALAR_FUNCTION_WRITER_BASE_HPP
