//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2001, 2002, 2003 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: ScalarFunctionWriterMedit.cpp,v 1.2 2006/11/22 23:47:09 delpinux Exp $

#include <ScalarFunctionWriterMedit.hpp>

#include <ScalarFunctionBase.hpp>
#include <FEMFunctionBase.hpp>

#include <Mesh.hpp>

#include <fstream>

void ScalarFunctionWriterMedit::
proceed() const
{
  std::string fileName = __fileName;
  fileName += ".bb";
  std::ofstream file(fileName.c_str());
  if (not(file)) {
    throw ErrorHandler(__FILE__,__LINE__,
		       "cannot open file '"+stringify(fileName)+"'",
		       ErrorHandler::normal);
  }

  file << "3 1 " << __mesh->numberOfVertices() << " 2" << __CR;

  switch (__function->type()) {
  case ScalarFunctionBase::femfunction: {
    const FEMFunctionBase& fem
      = static_cast<const FEMFunctionBase&>(*__function);
    if ((fem.baseMesh() == __mesh) and
	((fem.discretizationType() == DiscretizationType::LagrangianFEM1) or
	 (fem.discretizationType() == DiscretizationType::LagrangianFEM1))) {
      for (size_t i=0; i<fem.values().size(); ++i) {
	file << fem[i] << __CR;
      }
      break;
    } // if not continues the standard method
  }
  default: {
    const ScalarFunctionBase& f = *__function;
    for (size_t i=0; i<__mesh->numberOfVertices(); ++i) {
      const TinyVector<3,real_t>& X = __mesh->vertex(i);
      file << f(X) << __CR;
    }
  }
  }
}

ScalarFunctionWriterMedit::
ScalarFunctionWriterMedit(ConstReferenceCounting<ScalarFunctionBase> function,
			  ConstReferenceCounting<Mesh> mesh,
			  const std::string& fileName,
			  const std::string& CR)
  : ScalarFunctionWriterBase(function, mesh, fileName, CR)
{
  ;
}

ScalarFunctionWriterMedit::
~ScalarFunctionWriterMedit()
{
  ;
}
