//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2001, 2002, 2003 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: ScalarFunctionWriterRaw.cpp,v 1.1 2006/08/27 21:40:33 delpinux Exp $

#include <ScalarFunctionWriterRaw.hpp>

#include <ScalarFunctionBase.hpp>
#include <FEMFunctionBase.hpp>

#include <Mesh.hpp>

#include <fstream>

void ScalarFunctionWriterRaw::
proceed() const
{
  std::ofstream file(__fileName.c_str());
  if (file.bad()) {
    throw ErrorHandler(__FILE__,__LINE__,
		       "cannot open file '"
		       +stringify(__fileName)+"'",
		       ErrorHandler::normal);
  }

  switch (__function->type()) {
  case ScalarFunctionBase::femfunction: {
    const FEMFunctionBase& fem
      = static_cast<const FEMFunctionBase&>(*__function);
    if (fem.baseMesh() == __mesh) {
      for (size_t i=0; i<fem.values().size(); ++i) {
	file << fem[i] << __CR;
      }
      break;
    } // if not continues the standard method
  }
  default: {
    const ScalarFunctionBase& f = *__function;
    for (size_t i=0; i<__mesh->numberOfVertices(); ++i) {
      const TinyVector<3,real_t>& X = __mesh->vertex(i);
      file << f(X) << __CR;
    }
  }
  }
}

ScalarFunctionWriterRaw::
ScalarFunctionWriterRaw(ConstReferenceCounting<ScalarFunctionBase> function,
			ConstReferenceCounting<Mesh> mesh,
			const std::string& fileName,
			const std::string& CR)
  : ScalarFunctionWriterBase(function, mesh, fileName, CR)
{
  ;
}

ScalarFunctionWriterRaw::
~ScalarFunctionWriterRaw()
{
  ;
}
