//  This file is part of ff3d - http://www.freefem.org/ff3d
//  Copyright (C) 2001, 2002, 2003 Stphane Del Pino

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.

//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

//  $Id: XMLTag.hpp,v 1.1 2006/08/27 21:46:58 delpinux Exp $

#ifndef XML_TAG_HPP
#define XML_TAG_HPP

#include <ReferenceCounting.hpp>
#include <XMLAttribute.hpp>

#include <vector>
#include <iostream>
#include <string>

/**
 * @file   XMLTag.hpp
 * @author Stephane Del Pino
 * @date   Tue Aug  8 19:59:57 2006
 * 
 * @brief Simple embedding of basic XML tags
 */
class XMLTag
{
private:
  const std::string __name;
  std::vector<ReferenceCounting<XMLAttribute> > __attributes;

public:
  friend std::ostream& operator<< (std::ostream& os,
				   const XMLTag& xmlTag)
  {
    os << xmlTag.__name;
    for (size_t i=0; i<xmlTag.__attributes.size(); ++i) {
      os << ' ' << xmlTag.__attributes[i]->name() << "=\""
	 << xmlTag.__attributes[i]->value() << '"';
    }
    return os;
  }

  void add(ReferenceCounting<XMLAttribute> xmlAttribute)
  {
    __attributes.push_back(xmlAttribute);
  }

  const std::string& name() const
  {
    return __name;
  }

  XMLTag(const std::string& name)
    : __name(name)
  {
    ;
  }
};

#endif // XML_TAG_HPP
